@@ -3,6 +3,7 @@
 #![warn(missing_docs)]
 
 use std::cmp;
+use std::fmt;
 use std::io::Write;
 use std::mem;
 use std::path::Path;
@@ -37,6 +38,7 @@ lalrpop_util::lalrpop_mod!(
 
 #[derive(thiserror::Error, Debug)]
 /// Errors returned from the Assuan routines.
+#[non_exhaustive]
 pub enum Error {
     /// Handshake failed.
     #[error("Handshake failed: {0}")]
@@ -87,6 +89,16 @@ pub struct Client {
 }
 assert_send_and_sync!(Client);
 
+impl fmt::Debug for Client {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Client")
+         .field("r", &self.r)
+         .field("buffer", &self.buffer)
+         .field("done", &self.done)
+         .field("w", &self.w)
+         .finish()
+    }
+}
 enum WriteState {
     Ready(WriteHalf<IpcStream>),
     Sending(Pin<Box<dyn Future<Output = Result<WriteHalf<IpcStream>>>
