@@ -163,6 +163,7 @@ use assuan::Response;
 use assuan::escape;
 
 pub mod gnupg;
+pub use gnupg::Context;
 
 pub mod keyinfo;
 pub mod cardinfo;
@@ -179,6 +180,7 @@ pub use sequoia_ipc;
 
 #[derive(thiserror::Error, Debug)]
 /// Errors used in this module.
+#[non_exhaustive]
 pub enum Error {
     /// GnuPG's home directory doesn't exist.
     #[error("GnuPG's home directory ({0}) doesn't exist")]
@@ -292,10 +294,14 @@ fn trace_data_received(data: &[u8]) {
 }
 
 /// A connection to a GnuPG agent.
+#[derive(Debug)]
 pub struct Agent {
     socket_path: PathBuf,
     c: assuan::Client,
     pinentry_mode: Option<PinentryMode>,
+
+    /// Whether this is an restricted connection.
+    restricted: bool,
 }
 
 impl Deref for Agent {
@@ -359,34 +365,12 @@ impl Agent {
                 }
             }
 
-            if false {
-                // XXX: Currently, this will invoke gpgconf
-                // --create-socketdir, and fail if that fails.  It will
-                // also spew all sorts of output to stderr.
-                ctx.start("gpg-agent")?;
-            } else {
-                // In the mean time, manually start the agent.
-                let mut c = std::process::Command::new("gpgconf");
-                if let Some(h) = ctx.homedir() {
-                    c.env("GNUPGHOME", h);
-                    c.arg("--homedir").arg(&h.display().to_string());
-                }
-                c.arg("--launch").arg("gpg-agent");
-                c.status()?;
-            }
+            // Try to start gpg-agent.
+            ctx.start("gpg-agent")?;
 
             t!("Connecting to daemon");
             let path = ctx.socket("agent")?;
-            let mut connection = assuan::Client::connect(path).await?;
-            if TRACE.load(std::sync::atomic::Ordering::Relaxed) {
-                connection.trace_data_sent(Box::new(trace_data_sent));
-                connection.trace_data_received(Box::new(trace_data_received));
-            }
-            Ok(Agent {
-                socket_path: path.to_path_buf(),
-                c: connection,
-                pinentry_mode: None,
-            })
+            Agent::connect_to_agent(path).await
         }
 
         transaction(ctx).await.map_err(|e| {
@@ -446,11 +430,25 @@ impl Agent {
             connection.trace_data_sent(Box::new(trace_data_sent));
             connection.trace_data_received(Box::new(trace_data_received));
         }
-        Ok(Agent {
+
+        let mut agent = Agent {
             socket_path: socket_path.to_path_buf(),
             c: connection,
             pinentry_mode: None,
-        })
+            restricted: false,
+        };
+
+        // Check whether this is an restricted connection.
+        let restricted =
+            agent.send_simple("GETINFO restricted").await.is_ok();
+        agent.restricted = restricted;
+
+        Ok(agent)
+    }
+
+    /// Returns whether this is an restricted connection.
+    pub fn is_restricted(&self) -> bool {
+        self.restricted
     }
 
     /// Tells the gpg-agent to reload its configuration file.
@@ -1180,9 +1178,15 @@ impl Agent {
     }
 
     /// Computes options that we want to communicate.
-    fn options() -> Vec<String> {
+    fn options(&self) -> Vec<String> {
         use std::env::var;
 
+        if self.is_restricted() {
+            // We cannot set any of these options on restricted
+            // connections.
+            return vec![];
+        }
+
         let mut r = Vec::new();
 
         if let Ok(tty) = var("GPG_TTY") {
@@ -1574,7 +1578,7 @@ impl KeyPair {
         // Connect to the agent.
         let mut agent = Agent::connect_to_agent(&self.agent_socket).await?;
 
-        for option in Agent::options() {
+        for option in agent.options() {
             agent.send_simple(option).await?;
         }
 
@@ -1611,7 +1615,7 @@ impl KeyPair {
         // Connect to the agent.
         let mut agent = Agent::connect_to_agent(&self.agent_socket).await?;
 
-        for option in Agent::options() {
+        for option in agent.options() {
             agent.send_simple(option).await?;
         }
 
@@ -1649,7 +1653,7 @@ impl KeyPair {
         // operation.
         let mut agent = Agent::connect_to_agent(&self.agent_socket).await?;
 
-        for option in Agent::options() {
+        for option in agent.options() {
             agent.send_simple(option).await?;
         }
 
@@ -1706,14 +1710,16 @@ impl crypto::Signer for KeyPair {
     fn sign(&mut self, hash_algo: HashAlgorithm, digest: &[u8])
         -> openpgp::Result<openpgp::crypto::mpi::Signature>
     {
-        use tokio::runtime::{Handle, Runtime};
+        use tokio::runtime::{Handle, Builder};
 
         // See if the current thread is managed by a tokio
         // runtime.
         if Handle::try_current().is_err() {
             // Doesn't seem to be the case, so it is safe
             // to create a new runtime and block.
-            let rt = Runtime::new()?;
+            let rt = Builder::new_current_thread()
+                .enable_io()
+                .build()?;
             rt.block_on(self.sign_async(hash_algo, digest))
         } else {
             // It is!  We must not create a second runtime
@@ -1721,7 +1727,9 @@ impl crypto::Signer for KeyPair {
             // delegate this to a new thread.
             std::thread::scope(|s| {
                 s.spawn(move || {
-                    let rt = Runtime::new()?;
+                    let rt = Builder::new_current_thread()
+                        .enable_io()
+                        .build()?;
                     rt.block_on(self.sign_async(hash_algo, digest))
                 }).join()
             }).map_err(map_panic)?
@@ -1741,7 +1749,7 @@ impl KeyPair {
         // Connect to the agent and do the decryption operation.
         let mut agent = Agent::connect_to_agent(&self.agent_socket).await?;
 
-        for option in Agent::options() {
+        for option in agent.options() {
             agent.send_simple(option).await?;
         }
 
@@ -1814,14 +1822,16 @@ impl crypto::Decryptor for KeyPair {
                plaintext_len: Option<usize>)
         -> openpgp::Result<crypto::SessionKey>
     {
-        use tokio::runtime::{Handle, Runtime};
+        use tokio::runtime::{Handle, Builder};
 
         // See if the current thread is managed by a tokio
         // runtime.
         if Handle::try_current().is_err() {
             // Doesn't seem to be the case, so it is safe
             // to create a new runtime and block.
-            let rt = Runtime::new()?;
+            let rt = Builder::new_current_thread()
+                .enable_io()
+                .build()?;
             rt.block_on(self.decrypt_async(ciphertext, plaintext_len))
         } else {
             // It is!  We must not create a second runtime
@@ -1829,7 +1839,9 @@ impl crypto::Decryptor for KeyPair {
             // delegate this to a new thread.
             std::thread::scope(|s| {
                 s.spawn(move || {
-                    let rt = Runtime::new()?;
+                    let rt = Builder::new_current_thread()
+                        .enable_io()
+                        .build()?;
                     rt.block_on(self.decrypt_async(ciphertext, plaintext_len))
                 }).join()
             }).map_err(map_panic)?
